// Ira.cs
//
// Interactive program to compute the total accumulation 
// in an Individual Retirement Account under compound interest.  
// Assume that a deposit is made at the end of each year and that 
// interest is compounded annually.  

using System;

public class Ira
{
	public static void Main()
	{
		InputWrapper iw = new InputWrapper();
		double amount;  // annual deposit amount
		double rate;    // interest rate
		int years;      // number of years
		double total;   // total accumulation
		amount = iw.getDouble("amount: ");
		rate = iw.getDouble("rate: ");
		years = iw.getInt("years: ");

		// First calculate by formula
		total  =  amount * (Math.Pow(1 + rate, years) - 1) / rate;
		long total_in_cents = (long) Math.Round(total * 100);
		total = total_in_cents /100.0;
		Console.WriteLine("total = {0}", total);
		
		// Now calculate year by year and accumulate
		total = 0;
		double interest;
		Console.WriteLine("{0,4} {1,12} {2,12} {3,12}",
			"Year", "Amount", "Interest", "Total");
		for (int i = 1; i <= years; i++)
		{
			interest = total * rate;
			total += amount + interest;
			Console.WriteLine("{0, -4} {1, 12:C} {2, 12:C} {3, 12:C}",
				i, amount, interest, total);
		}
	}
}
