// VendorDriver.cs

using System;

public class VendorDriver
{
	private static InputWrapper iw = new InputWrapper(); 
	public static void Loop()
	{
		Console.WriteLine("Your inventory:");
		ShowList(Game.CurrentVendor.GetProducts());
		Console.WriteLine("Enter command, logout when done");
		string cmd = iw.getString(Game.CurrentVendor.Prompt());
		while (true)
		{
			try
			{
				if (cmd.Equals("logout"))
				{
					string name = Game.CurrentVendor.Name;
					string status = Game.players.Logout(name);
					if (status == "OK")
					{
						Game.CurrentVendor = null;
						Console.WriteLine("{0} has logged out", name);
						return;
					}
					else
						Console.WriteLine(status);
				}
				else if (cmd.Equals("buy"))
				{
					string prodName = iw.getString("product name: ");
					int qty = iw.getInt("quantity: ");
					string status = Game.VendorBuy(prodName, qty);
					Console.WriteLine(status);
				}
				else if (cmd.Equals("add"))
				{
					string prodName = iw.getString("product name: ");
					// Make sure product is available from wholesaler
					Product prod = Game.whole.Products.FindProduct(prodName);
					if (prod != null)
					{
						int qty = 0;
						decimal price = iw.getDecimal("price: ");
						string status = Game.CurrentVendor.Products.AddProduct(prodName,qty,price);
						Console.WriteLine(status);
					}
					else
						Console.WriteLine("Product not available at wholesale");
				}
				else if (cmd.Equals("delete"))
				{
					string prodName = iw.getString("product name: ");
					string status = Game.CurrentVendor.Products.DeleteProduct(prodName);
					Console.WriteLine(status);
				}
				else if (cmd.Equals("mylist"))
				{
					string name = Game.CurrentVendor.Name;
					decimal bal;
					string status = Game.players.GetBalance(name, out bal);
					if (status == "OK")
					{
						Console.WriteLine("Balance = {0:C}", bal);
						ShowList(Game.CurrentVendor.GetProducts());
					}
				}
				else if (cmd.Equals("wlist"))
				{
					ShowList(Game.whole.GetProducts());
				}
				else
					help();
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception: {0}", e.Message);
			}
			cmd = iw.getString(Game.CurrentVendor.Prompt());
		}
	}

	private static void ShowList(Product[] list)
	{
		foreach (Product prod in list)
			Console.WriteLine(prod.Show());
	}

	private static void help()
	{
		Console.WriteLine("The following commands are available:");
		Console.WriteLine("\tlogout  -- logout current vendor");
		Console.WriteLine("\tbuy     -- buy from wholesaler");
		Console.WriteLine("\tadd     -- add a product to product list");
		Console.WriteLine("\tdelete  -- delete a product from product list");
		Console.WriteLine("\tmylist  -- list products of current vendor");
		Console.WriteLine("\twlist   -- list products of wholesaler");
	}
}