// Player.cs

using System;
using System.Collections;

public enum PlayerRole
{
	Shopper = 1,
	Vendor = 2
}

abstract public class Player
{
	public string Name;	
	private decimal m_balance;
	public bool Active;
	public string m_password;
	public decimal Balance
	{
		get
		{
			return m_balance;
		}
		set
		{
			m_balance = value;
		}
	}
	public Player()
	{
	}
	public Player(string n, string pwd, decimal bal)
	{
		Name = n;
		m_password = pwd;
		m_balance = bal;
		Active = false;
	}
	public Player(string n, string pwd, decimal bal, bool isActive)
	{
		Name = n;
		m_password = pwd;
		m_balance = bal;
		Active = isActive;
	}
	public bool CheckPassword(string pwd)
	{
		if (pwd == m_password)
			return true;
		else
			return false;
	}
	virtual public string Show()
	{
		string strActive = (Active ? "Active    " : "Not Active");
		string str = string.Format("{0, -10}  {1, 10:C}", Name, Balance);
		return Prompt() + "   " + str + "   " + strActive;
	}
	virtual public string Prompt()
	{
		return "> ";
	}
}

public class PlayerList : IPlayerAdmin, IPlayer, IVendorAdmin
{
	private const int MAXPLAYER = 10;
	private static int m_next = 0;
	private Player[] m_list = new Player[MAXPLAYER];
	public int Count
	{
		get
		{
			return m_next;
		}
	}

	private int FindIndex(string name)
	{
		for (int i = 0; i < Count; i++)
		{
			Player p = m_list[i];
			if (p.Name == name)
			{
				return i;
			}
		}
		return -1;		// not found
	}

	public Player FindPlayer(string name)
	{
		int index = FindIndex(name);
		if (index == -1)
			return null;
		else
			return m_list[index];
	}

	public string AddPlayer(string name, string pwd, decimal bal, int roleId)
	{
		Player play = null;
		if (roleId == (int) PlayerRole.Shopper)
			play = new Shopper(name, pwd, bal);
		else if (roleId == (int) PlayerRole.Vendor)
			play = new Vendor(name, pwd, bal);
		if (play == null)
			return "Illegal role id";
		else
		{
			AddPlayer(play);
			return "OK";
		}
	}

	private string AddPlayer(Player p)
	{
		if (m_next >= MAXPLAYER)
			return "Too many players";
		else if (FindIndex(p.Name) != -1)
			return "Already on list";
		else
		{
			m_list[m_next++] = p;
			return "OK";
		}
	}

	public string DeletePlayer(string name)
	{
		if (Count == 0)
			return "List is empty";
		int index = FindIndex(name);
		if (index == -1)
			return "Player not found";
		// move players to fill in
		for (int i = index; i < Count - 1; i++)
			m_list[i] = m_list[i+1];
		m_next--;
		return "OK";
	}

	public IList GetPlayers()
	{
		Player[] list = new Player[Count];
		for (int i = 0; i < Count; i++)
			list[i] = m_list[i];
		return list;
	}

	public string Login(string name, string pwd, out Player vPlayer)
	{
		vPlayer = null;
		int index = FindIndex(name);
		if (index == -1)
			return "Player not found";
		Player p = m_list[index];
		if (!p.CheckPassword(pwd))
			return "Illegal password";
		else if (p.Active)
			return "Player already active";
		else
		{
			p.Active = true;
			vPlayer = p;
			return "OK";
		}
	}

	public string Logout(string name)
	{
		int index = FindIndex(name);
		if (index == -1)
			return "Player not found";
		Player p = m_list[index];
		if (p.Active)
		{
			p.Active = false;
			return "OK";
		}
		else
			return "Player inactive";
	}

	public string ChangeBalance(string name, decimal delta)
	{
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		else
		{
			play.Balance += delta;
			return "OK";
		}
	}

	public string GetBalance(string name, out decimal bal)
	{
		bal = 0m;
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		else
		{
			bal = play.Balance;
			return "OK";
		}
	}

	public string SetBalance(string name, decimal bal)
	{
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		else
		{
			play.Balance = bal;
			return "OK";
		}
	}

	public string FindUrl(string name)
	{
		foreach (Player p in m_list)
		{
			if (p.GetType() == typeof(Vendor))
			{
				Vendor v = (Vendor) p;
				if (v.Name == name)
					return v.Url;
			}
		}
		return null;
	}

	public string AddUrl(string name, string url)
	{
		Player play = FindPlayer(name);
		Vendor v = play as Vendor;
		if (v != null)
		{
			v.Url = url;
			return "OK";
		}
		else
			return "Player is not a vendor";
	}

	public string DeleteUrl(string name)
	{
		return "OK";
	}
}
