// ThreadList.cs

using System;
using System.Collections;
using System.Threading;

public class ThreadInfo
{
	private int m_delta;
	private Thread m_thread;
	public ThreadInfo(Thread t, int delta)
	{
		m_thread = t;
		m_delta = delta;
	}
	public string Name
	{
		get
		{
			return m_thread.Name;
		}
	}
	public Thread t
	{
		get
		{
			return m_thread;
		}
	}
	override public string ToString()
	{
		return string.Format("{0, -10}  {1, 5}", m_thread.Name, m_delta)
			+ "   "  + StateString(t.ThreadState);
	}
	private string StateString(ThreadState nstate)
	{
		string state = "";
		if (nstate == 0)
			state = "Running";
		else
		{
			if ((nstate & ThreadState.Unstarted) != 0)
				state = "Unstarted " + state;
			if ((nstate & ThreadState.Stopped) != 0)
				state = "Stopped " + state;
			if ((nstate & ThreadState.WaitSleepJoin) != 0)
				state = "WaitSleepJoin " + state;
			if ((nstate & ThreadState.Suspended) != 0)
				state = "Suspended " + state;
			if ((nstate & ThreadState.AbortRequested) != 0)
				state = "AbortRequested " + state;
		}
		return state;
	}
}

public class ThreadList
{
	private ArrayList m_threads = new ArrayList();
	public void Add(Thread t, int ticks)
	{
		ThreadInfo ti = new ThreadInfo(t, ticks);
		m_threads.Add(ti);
	}
	public Thread Find(string name)
	{
		foreach (ThreadInfo ti in m_threads)
		{
			if (ti.Name == name)
				return ti.t;
		}
		return null;
	}
	public IList GetThreads()
	{
		return m_threads;
	}
}