// Player.cs

using System;
using System.Collections;

public enum PlayerRole
{
	Shopper = 1,
	Vendor = 2
}

public class Player
{
	public string Name;	
	private decimal m_balance;
	public bool Active;
	public string m_password;
	public PlayerRole Role;
	public decimal Balance
	{
		get
		{
			return m_balance;
		}
		set
		{
			m_balance = value;
		}
	}
	public Player()
	{
	}
	public Player(string n, string pwd, decimal bal, PlayerRole role)
	{
		Name = n;
		m_password = pwd;
		m_balance = bal;
		Active = false;
		Role = role;
	}
	public bool CheckPassword(string pwd)
	{
		if (pwd == m_password)
			return true;
		else
			return false;
	}
	override public string ToString()
	{
		string strActive = (Active ? "Active    " : "Not Active");
		string str = string.Format("{0, -10}  {1, 10:C}", Name, Balance);
		string strRole;
		if (Role == PlayerRole.Shopper)
			strRole = "Shopper";
		else
			strRole = "Vendor ";
		return Prompt() + "   " + str + "   " + strActive + "   " + strRole;
	}
	virtual public string Prompt()
	{
		return "> ";
	}
}

public class PlayerList : IPlayerAdmin, IPlayer, IPlayerEvents
{
	private ArrayList m_list = new ArrayList();
	public event PlayerEventHandler Disqualified;
	public int Count
	{
		get
		{
			return m_list.Count;
		}
	}

	public Player FindPlayer(string name)
	{
		foreach (Player play in m_list)
		{
			if (play.Name == name)
				return play;
		}
		return null;
	}

	public string AddPlayer(string name, string pwd, decimal bal, int roleId)
	{
		PlayerRole role;
		if (roleId == (int) PlayerRole.Shopper || roleId == (int) PlayerRole.Vendor)
		{
			role = (PlayerRole) roleId;
			AddPlayer(new Player(name, pwd, bal, role));
			return "OK";
		}
		else
			return "Illegal role id";
	}

	private string AddPlayer(Player p)
	{
		if (FindPlayer(p.Name) != null)
			return "Already on list";
		else
		{
			m_list.Add(p);
			return "OK";
		}
	}

	public string DeletePlayer(string name)
	{
		if (Count == 0)
			return "List is empty";
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		m_list.Remove(play);
		return "OK";
	}

	public IList GetPlayers()
	{
		return m_list;
	}

	public string Login(string name, string pwd, out Player vPlayer)
	{
		vPlayer = null;
		Player p = FindPlayer(name);
		if (p == null)
			return "Player not found";
		if (!p.CheckPassword(pwd))
			return "Illegal password";
		else if (p.Active)
			return "Player already active";
		else
		{
			p.Active = true;
			vPlayer = p;
			return "OK";
		}
	}

	public string Logout(string name)
	{
		Player p = FindPlayer(name);
		if (p == null)
			return "Player not found";
		if (p.Active)
		{
			p.Active = false;
			return "OK";
		}
		else
			return "Player inactive";
	}

	public string ChangeBalance(string name, decimal delta)
	{
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		else
		{
			play.Balance += delta;
			if (play.Balance < 0m)
				Disqualified(this, new PlayerEventArgs(name, play.Balance));
			return "OK";
		}
	}

	public string GetBalance(string name, out decimal bal)
	{
		bal = 0m;
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		else
		{
			bal = play.Balance;
			return "OK";
		}
	}

	public string SetBalance(string name, decimal bal)
	{
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		else
		{
			play.Balance = bal;
			if (play.Balance < 0m)
				Disqualified(this, new PlayerEventArgs(name, play.Balance));
			return "OK";
		}
	}
}
