// PlayerDriver.cs

using System;

public class PlayerDriver
{
  	private static InputWrapper iw = new InputWrapper(); 
	private static IPlayer iplay;
	private static IPlayerAdmin iplayadm;
	private static IVendorAdmin ivendadm;
	public static void Loop()
	{
		SetInterfaces();
		Console.WriteLine("Enter command, quit to exit");
		string cmd = iw.getString("player> ");
		while (! cmd.Equals("quit"))
		{
			try
			{
				if (cmd.Equals("login"))
				{
					string name = iw.getString("name: ");
					string pwd = iw.getString("password: ");
					Player p;
					string status = iplay.Login(name, pwd, out p);
					if (status == "OK")
					{
						Console.WriteLine("Welcome, {0}", p.Name);
						Console.WriteLine("Your balance is {0:C}", p.Balance);
					}
					else
						Console.WriteLine(status);
				}
				else if (cmd.Equals("logout"))
				{
					string name = iw.getString("name: ");
					string status = iplay.Logout(name);
					if (status == "OK")
						Console.WriteLine("{0} has logged out", name);
					else
						Console.WriteLine(status);
				}
				else if (cmd.Equals("add"))
				{
					string name = iw.getString("name: ");
					string pwd = iw.getString("password: ");
					decimal bal = iw.getDecimal("balance: ");
					string status;
					int roleid = iw.getInt("role id (1 = Shopper, 2 = Vendor): ");
					if (roleid == (int) PlayerRole.Vendor)
					{
						status = iplayadm.AddPlayer(name, pwd, bal, roleid);
						if (status == "OK")
						{
							string url = iw.getString("url: ");
							status = ivendadm.AddUrl(name, url);
						}
					}
					else if (roleid == (int) PlayerRole.Shopper)
					{
						status = iplayadm.AddPlayer(name, pwd, bal, roleid);
					}
					else
						status = "Illegal role id";
					if (status == "OK")
					{
						Console.WriteLine("{0} has been added", name);
					}
					else
						Console.WriteLine(status);
				}
				else if (cmd.Equals("delete"))
				{
					string name = iw.getString("name: ");
					Player play = iplayadm.FindPlayer(name);
					if (play == null)
						Console.WriteLine("Player not found");
					else
					{
						string status = iplayadm.DeletePlayer(name);
						Console.WriteLine(status);
					}
				}
				else if (cmd.Equals("find"))
				{
					string name = iw.getString("name: ");
					Player play = iplayadm.FindPlayer(name);
					if (play == null)
						Console.WriteLine("Player not found");
					else
						Console.WriteLine(play.ToString());
				}
				else if (cmd.Equals("list"))
				{
					Console.WriteLine("Count = {0}", iplayadm.Count);
					DisplayPlayers();
				}
				else if (cmd.Equals("balance"))
				{
					string name = iw.getString("name: ");
					Player play = iplayadm.FindPlayer(name);
					if (play == null)
						Console.WriteLine("Player not found");
					else
					{
						Console.WriteLine("balance = {0:C}", play.Balance);
						decimal bal = iw.getDecimal("new balance: ");
						string status = iplay.SetBalance(name, bal);
						if (status == "OK")
						{
							decimal newbal;
							status = iplay.GetBalance(name, out newbal);
							if (status != "OK")
								Console.WriteLine("status");
							else
								Console.WriteLine("new balance = {0:C}", newbal);
						}
					}
				}
				else if (cmd.Equals("change"))
				{
					string name = iw.getString("name: ");
					Player play = iplayadm.FindPlayer(name);
					if (play == null)
						Console.WriteLine("Player not found");
					else
					{
						Console.WriteLine("balance = {0:C}", play.Balance);
						decimal delta = iw.getDecimal("delta: ");
						string status = iplay.ChangeBalance(name, delta);
						if (status == "OK")
						{
							decimal newbal;
							status = iplay.GetBalance(name, out newbal);
							if (status != "OK")
								Console.WriteLine("status");
							else
								Console.WriteLine("new balance = {0:C}", newbal);
						}
					}
				}
				else
					help();
			}
			catch (Exception e)
			{
				Console.WriteLine("Exception: {0}", e.Message);
			}
			cmd = iw.getString("player> ");
		}
	}

	public static void DisplayPlayers()
	{
		foreach (Player p in iplayadm.GetPlayers())
		{
			Console.WriteLine(p);
		}							   
	}

	private static void SetInterfaces()
	{
		iplay = Game.players;
		iplayadm = Game.players;
		ivendadm = Game.players;
	}

	private static void help()
	{
		Console.WriteLine("The following commands are available:");
		Console.WriteLine("\tlogin   -- login a player");
		Console.WriteLine("\tlogout  -- logout a player");
		Console.WriteLine("\tadd     -- add a new player");
		Console.WriteLine("\tdelete  -- delete a player");
		Console.WriteLine("\tlist    -- list player information");
		Console.WriteLine("\tbalance -- assign a new balance");
		Console.WriteLine("\tchange  -- change the balance");
		Console.WriteLine("\tquit    -- exit the driver");
	}
}