// Player.cs

using System;
using System.Data;
using System.Data.OleDb;
using System.Collections;

public enum PlayerRole
{
	Shopper = 1,
	Vendor = 2
}

public class Player
{
	public string Name;	
	protected decimal m_balance;
	public bool Active;
	public string m_password;
	public decimal Balance
	{
		get
		{
			return m_balance;
		}
		set
		{
			m_balance = value;
		}
	}
	public Player()
	{
	}
	public Player(string n, string pwd, decimal bal)
	{
		Name = n;
		m_password = pwd;
		m_balance = bal;
		Active = false;
	}
	public Player(string n, string pwd, decimal bal, bool isActive)
	{
		Name = n;
		m_password = pwd;
		m_balance = bal;
		Active = isActive;
	}
	public bool CheckPassword(string pwd)
	{
		if (pwd == m_password)
			return true;
		else
			return false;
	}
	virtual public string Show()
	{
		string strActive = (Active ? "Active" : "Not Active");
		return Name + "\t" + Balance + "\t" + strActive;
	}
	virtual public string Prompt()
	{
		return "> ";
	}
}

public class PlayerDb :IPlayerAdmin, IPlayer
{
	public static string connStr = 
		"Provider=Microsoft.Jet.OLEDB.4.0;"
		+ "Data Source=C:\\OI\\CSharpEx\\Databases\\Game.mdb";
	public static OleDbConnection conn;
	private IVendorAdmin m_vendorAdmin;

	public PlayerDb()
	{
		conn = new OleDbConnection();
		conn.ConnectionString = connStr;
		m_vendorAdmin = new VendorData();
	}

	public void ShowOpen(string label)
	{
		Console.WriteLine(label + " : Database state: " + conn.State.ToString());
	}

	public int Count
	{
		get
		{
			string query = "select count(*) from Players";
			OleDbCommand cmd = new OleDbCommand(query, conn);
			conn.Open();
			int result = (int) cmd.ExecuteScalar();
			conn.Close();
			return result;
		}
	}

	public string AddPlayer(string name, string pwd, decimal bal, int roleId)
	{
		string query = "insert into Players values (?, ?, ?, False, ?)";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Name", name);
		cmd.Parameters.Add("@Password", pwd);
		cmd.Parameters.Add("@Balance", bal);
		cmd.Parameters.Add("@RoleId", roleId);

		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public Player FindPlayer(string name)
	{
		string query = "select * from Players where Name = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Name", name);
		conn.Open();
		IDataReader reader = cmd.ExecuteReader();
		Player play;
		if (reader.Read())
		{
			string pwd = (string) reader["Password"];
			decimal bal = (decimal) reader["Balance"];
			bool isActive = (bool) reader["Active"];
			PlayerRole roleId = (PlayerRole) reader["RoleId"];
			if (roleId == PlayerRole.Shopper)
				play = new Shopper(name, pwd, bal, isActive);
			else if (roleId == PlayerRole.Vendor)
				play = new Vendor(name, pwd, bal, isActive, "someurl.com");
			else 
				play = null;
		}
		else
			play = null;
		reader.Close();
		conn.Close();
		return play;
	}

	public string DeletePlayer(string name)
	{
		if (Count == 0)
			return "List is empty";
		string query = "delete from Players where Name = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Name", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public IList GetPlayers()
	{
		string query = "select * from Players";
		OleDbCommand cmd = new OleDbCommand(query, conn);

		Player[] array = new Player[Count];

		conn.Open();
		IDataReader reader = cmd.ExecuteReader();
		int i = 0;
		while (reader.Read())
		{
			Player play;
			string name = (string) reader["Name"];
			string pwd = (string) reader["Password"];
			decimal bal = (decimal) reader["Balance"];
			bool isActive = (bool) reader["Active"];
			PlayerRole roleId = (PlayerRole) reader["RoleId"];
			if (roleId == PlayerRole.Shopper)
				play = new Shopper(name, pwd, bal, isActive);
			else if (roleId == PlayerRole.Vendor)
			{
				string url = m_vendorAdmin.FindUrl(name);
				if (url == null)
					url = "";
				play = new Vendor(name, pwd, bal, isActive, url);
			}
			else
				play = new Player(name, pwd, bal, isActive);
			array[i++] = play;
		}
		reader.Close();
		conn.Close();
		return array;
	}

	public string Login(string name, string pwd, out Player play)
	{
		play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		if (!play.CheckPassword(pwd))
			return "Illegal password";
		else if (play.Active)
			return "Player already active";
		else
			return SetActive(name, true);
	}

	private string SetActive(string name, bool isActive)
	{
		string query = "update Players set Active = ? where Name = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Active", isActive);
		cmd.Parameters.Add("@Name", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public string ChangeBalance(string name, decimal delta)
	{
		string query = "update Players set Balance = Balance + ? where Name = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Delta", delta);
		cmd.Parameters.Add("@Name", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public string GetBalance(string name, out decimal bal)
	{
		string query = "select Balance from Players where Name = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Name", name);
		conn.Open();
		bal = 0m;
		object obj = cmd.ExecuteScalar();
		conn.Close();
		if (obj == null)
			return "Player not found";
		else
		{
			bal = (decimal) obj;
			return "OK";
		}
	}

	public string SetBalance(string name, decimal bal)
	{
		string query = "update Players set Balance = ? where Name = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Balance", bal);
		cmd.Parameters.Add("@Name", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public string Logout(string name)
	{
		Player play = FindPlayer(name);
		if (play == null)
			return "Player not found";
		if (play.Active)
			return SetActive(name, false);
		else
			return "Player inactive";
	}
}