// Product.cs

using System;
using System.Data;
using System.Data.OleDb;
using System.Collections;

public class Product : Item
{
	public decimal Price;
	public Product(string n, int qty, decimal p) : base(n, qty)
	{
		Price = p;
	}
	new public string Show()
	{
		return base.Show() + string.Format("  {0,10:C}", Price);
	}
}

public class ProductDb : IProductList, IVendorName
{
	private string m_vendorName;

	public static string connStr = 
		"Provider=Microsoft.Jet.OLEDB.4.0;"
		+ "Data Source=C:\\OI\\CSharpEx\\Databases\\Game.mdb";
	public static OleDbConnection conn;

	public ProductDb(string vendorName)
	{
		m_vendorName = vendorName;
		conn = new OleDbConnection();
		conn.ConnectionString = connStr;
	}

	public string VendorName
	{
		get
		{
			return m_vendorName;
		}
		set
		{
			m_vendorName = value;
		}
	}
	public int Count
	{
		get
		{
			string query = "select count(*) from Products where VendorName = ?";
			OleDbCommand cmd = new OleDbCommand(query, conn);
			cmd.Parameters.Add("@VendorName", VendorName);
			conn.Open();
			int result = (int) cmd.ExecuteScalar();
			conn.Close();
			return result;
		}
	}
	public string AddProduct(string name, int qty, decimal price)
	{
		if (price <= 0m)
			return "Price must be positive";

		string query = "insert into Products values (?, ?, ?, ?)";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@VendorName", VendorName);
		cmd.Parameters.Add("@ProductrName", name);
		cmd.Parameters.Add("@Quantity", qty);
		cmd.Parameters.Add("@Price", price);

		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public string DeleteProduct(string name)
	{
		if (Count == 0)
			return "List is empty";
		string query = "delete from Products where VendorName = ? and ProductName = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@VendorName", VendorName);
		cmd.Parameters.Add("@ProductName", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}
	public Product FindProduct(string name)
	{
		string query = "select * from Products where VendorName = ? and ProductName = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@VendorName", VendorName);
		cmd.Parameters.Add("@ProductName", name);
		conn.Open();
		IDataReader reader = cmd.ExecuteReader();
		Product prod;
		if (reader.Read())
		{
			string prodName = (string) reader["ProductName"];
			int qty = (int) reader["Quantity"];
			decimal price = (decimal) reader["Price"];
			prod = new Product(prodName, qty, price);
		}
		else
			prod = null;
		reader.Close();
		conn.Close();
		return prod;
	}

	public string ChangeQuantity(string name, int delta)
	{
		string query = "update Products set Quantity = Quantity + ? where VendorName = ? and ProductName = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Delta", delta);
		cmd.Parameters.Add("@VendorName", VendorName);
		cmd.Parameters.Add("@ProductName", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public string ChangePrice(string name, decimal delta)
	{
		string query = "update Products set Price = Price + ? where VendorName = ? and ProductName = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@Delta", delta);
		cmd.Parameters.Add("@VendorName", VendorName);
		cmd.Parameters.Add("@ProductName", name);
		conn.Open();
		int numrow = cmd.ExecuteNonQuery();
		conn.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public IList GetProducts()
	{
		string query = "select * from Products where VendorName = ?";
		OleDbCommand cmd = new OleDbCommand(query, conn);
		cmd.Parameters.Add("@VendorName", VendorName);

		// Invoking Count property after ExecuteReader is a problem:
		// data reader already open
		// TODO: Find more robust solution for opening/closing DB resources
        Product[] array = new Product[Count];

		conn.Open();
		IDataReader reader = cmd.ExecuteReader();
		int i = 0;
		while (reader.Read())
		{
			string prodName = (string) reader["ProductName"];
			int qty = (int) reader["Quantity"];
			decimal price = (decimal) reader["Price"];
			array[i++] = new Product(prodName, qty, price);
		}
		reader.Close();
		conn.Close();
		return array;
	}
}