// Vendor.cs

using System;
using System.Data;
using System.Data.OleDb;
using System.Collections;

public class Vendor : Player
{
	private ProductDb m_list;
	public string Url;
	public Vendor(string name, string pwd, decimal bal, string vUrl) : 
		base(name, pwd, bal)
	{
		Url = vUrl;
		m_list = new ProductDb(name);
	}
	public Vendor(string name, string pwd, decimal bal, bool isActive, string vUrl) : 
		base(name, pwd, bal, isActive)
	{
		Url = vUrl;
		m_list = new ProductDb(name);
	}
	public Vendor(Player p, string vUrl)
	{
		Name = p.Name;
		m_password = p.m_password;
		m_balance = p.Balance;
		Active = p.Active;
		Url = vUrl;
		m_list = new ProductDb(Name);
	}
	override public string Prompt()
	{
		return "V> ";
	}
	override public string Show()
	{
		return base.Show() + "\t" + Url;
	}
	public string Sell(string prodName, int qty, out decimal price)
	{
		price = 0m;
		Product prod = m_list.FindProduct(prodName);
		if (prod == null)
			return "Product not found";
		if (prod.Quantity < qty)
			return "Insufficient quantity";
		string status = m_list.ChangeQuantity(prodName, -qty);
		if (status != "OK")
			return status;
		Game.players.ChangeBalance(Name, qty * prod.Price);
		price = prod.Price;
		return "OK";
	}
	public IList GetList()
	{
		return m_list.GetProducts();
	}
	public ProductDb Products
	{
		get
		{
			return m_list;
		}
	}
}

public class VendorData : IVendorAdmin, IVendorInfo
{
	public static string connStr = 
		"Provider=Microsoft.Jet.OLEDB.4.0;"
		+ "Data Source=C:\\OI\\CSharpEx\\Databases\\Game.mdb";
	public static OleDbConnection conn;

	public VendorData()
	{
		conn = new OleDbConnection();
		conn.ConnectionString = connStr;
	}

	public void ShowOpen(string label)
	{
		Console.WriteLine(label + " : Database state: " + conn.State.ToString());
	}

	OleDbConnection GetOpenConnection()
	{
		OleDbConnection conn = new OleDbConnection(connStr);
		conn.Open();
		return conn;
	}

	public string FindUrl(string name)
	{
		string query = "select Url from Vendors where Name = ?";
		string url;

		OleDbConnection con = GetOpenConnection();

		OleDbCommand cmd = new OleDbCommand(query, con);
		cmd.Parameters.Add("@Name", name);
		IDataReader reader = cmd.ExecuteReader();
		if (reader.Read())
			url = (string) reader["Url"];
		else
			url = null;
		reader.Close();
		con.Close();
		return url;
	}

	public string AddUrl(string name, string url)
	{
		string query = "insert into Vendors values (?, ?)";
		OleDbConnection con = GetOpenConnection();
		OleDbCommand cmd = new OleDbCommand(query, con);
		cmd.Parameters.Add("@Name", name);
		cmd.Parameters.Add("@Url", url);

		int numrow = cmd.ExecuteNonQuery();
		con.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public string DeleteUrl(string name)
	{
		string query = "delete from Vendors where Name = ?";
		OleDbConnection con = GetOpenConnection();
		OleDbCommand cmd = new OleDbCommand(query, con);
		cmd.Parameters.Add("@Name", name);
		int numrow = cmd.ExecuteNonQuery();
		con.Close();
		if (numrow == 1)
			return "OK";
		else
			return numrow + " rows updated";
	}

	public Vendor FindVendor(string url)
	{
		// First find the player name
		string query = "select Name from Vendors where Url = ?";
		OleDbConnection con = GetOpenConnection();
		OleDbCommand cmd = new OleDbCommand(query, con);
		cmd.Parameters.Add("@Url", url);
		IDataReader reader = cmd.ExecuteReader();
		if (!reader.Read())
		{
			// not found
			reader.Close();
			con.Close();
			return null;
		}
		string name = (string) reader["Name"];

		// Now find the player and construct a Vendor object
		Vendor vend;
		IPlayerAdmin prods = new PlayerDb();
		Player play = prods.FindPlayer(name);
		if (play == null)
			vend = null;
		else
			vend = new Vendor(play, url);

		reader.Close();
		con.Close();
		return vend;
	}
}
